@@ -1,59 +1,55 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
 import sys
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
-
-from gunicorn.http.errors import NoMoreData, ChunkMissingTerminator, \
-InvalidChunkSize
+from gunicorn.http.errors import (NoMoreData, ChunkMissingTerminator,
+        InvalidChunkSize)
+from gunicorn import six
 
 class ChunkedReader(object):
     def __init__(self, req, unreader):
         self.req = req
         self.parser = self.parse_chunked(unreader)
-        self.buf = StringIO()
-    
+        self.buf = six.BytesIO()
+
     def read(self, size):
-        if not isinstance(size, (int, long)):
+        if not isinstance(size, six.integer_types):
             raise TypeError("size must be an integral type")
         if size <= 0:
             raise ValueError("Size must be positive.")
         if size == 0:
-            return ""
+            return b""
 
         if self.parser:
             while self.buf.tell() < size:
                 try:
-                    self.buf.write(self.parser.next())
+                    self.buf.write(six.next(self.parser))
                 except StopIteration:
                     self.parser = None
                     break
 
         data = self.buf.getvalue()
         ret, rest = data[:size], data[size:]
-        self.buf.truncate(0)
+        self.buf = six.BytesIO()
         self.buf.write(rest)
         return ret
-    
+
     def parse_trailers(self, unreader, data):
-        buf = StringIO()
+        buf = six.BytesIO()
         buf.write(data)
-        
-        idx = buf.getvalue().find("\r\n\r\n")
-        done = buf.getvalue()[:2] == "\r\n"
+
+        idx = buf.getvalue().find(b"\r\n\r\n")
+        done = buf.getvalue()[:2] == b"\r\n"
         while idx < 0 and not done:
             self.get_data(unreader, buf)
-            idx = buf.getvalue().find("\r\n\r\n")
-            done = buf.getvalue()[:2] == "\r\n"
+            idx = buf.getvalue().find(b"\r\n\r\n")
+            done = buf.getvalue()[:2] == b"\r\n"
         if done:
             unreader.unread(buf.getvalue()[2:])
-            return ""
+            return b""
         self.req.trailers = self.req.parse_headers(buf.getvalue()[:idx])
         unreader.unread(buf.getvalue()[idx+4:])
 
@@ -71,24 +67,24 @@ class ChunkedReader(object):
             rest = rest[size:]
             while len(rest) < 2:
                 rest += unreader.read()
-            if rest[:2] != '\r\n':
+            if rest[:2] != b'\r\n':
                 raise ChunkMissingTerminator(rest[:2])
-            (size, rest) = self.parse_chunk_size(unreader, data=rest[2:])          
+            (size, rest) = self.parse_chunk_size(unreader, data=rest[2:])
 
     def parse_chunk_size(self, unreader, data=None):
-        buf = StringIO()
+        buf = six.BytesIO()
         if data is not None:
             buf.write(data)
 
-        idx = buf.getvalue().find("\r\n")
+        idx = buf.getvalue().find(b"\r\n")
         while idx < 0:
             self.get_data(unreader, buf)
-            idx = buf.getvalue().find("\r\n")
+            idx = buf.getvalue().find(b"\r\n")
 
         data = buf.getvalue()
         line, rest_chunk = data[:idx], data[idx+2:]
-    
-        chunk_size = line.split(";", 1)[0].strip()
+
+        chunk_size = line.split(b";", 1)[0].strip()
         try:
             chunk_size = int(chunk_size, 16)
         except ValueError:
@@ -112,26 +108,26 @@ class LengthReader(object):
     def __init__(self, unreader, length):
         self.unreader = unreader
         self.length = length
-    
+
     def read(self, size):
-        if not isinstance(size, (int, long)):
+        if not isinstance(size, six.integer_types):
             raise TypeError("size must be an integral type")
-        
+
         size = min(self.length, size)
         if size < 0:
             raise ValueError("Size must be positive.")
         if size == 0:
-            return ""
-        
-        
-        buf = StringIO()
+            return b""
+
+
+        buf = six.BytesIO()
         data = self.unreader.read()
         while data:
             buf.write(data)
             if buf.tell() >= size:
                 break
             data = self.unreader.read()
-        
+
         buf = buf.getvalue()
         ret, rest = buf[:size], buf[size:]
         self.unreader.unread(rest)
@@ -141,17 +137,24 @@ class LengthReader(object):
 class EOFReader(object):
     def __init__(self, unreader):
         self.unreader = unreader
-        self.buf = StringIO()
+        self.buf = six.BytesIO()
         self.finished = False
-    
+
     def read(self, size):
-        if not isinstance(size, (int, long)):
+        if not isinstance(size, six.integer_types):
             raise TypeError("size must be an integral type")
         if size < 0:
             raise ValueError("Size must be positive.")
-        if size == 0 or self.finished:
-            return ""
-        
+        if size == 0:
+            return b""
+
+        if self.finished:
+            data = self.buf.getvalue()
+            ret, rest = data[:size], data[size:]
+            self.buf = six.BytesIO()
+            self.buf.write(rest)
+            return ret
+
         data = self.unreader.read()
         while data:
             self.buf.write(data)
@@ -161,46 +164,46 @@ class EOFReader(object):
 
         if not data:
             self.finished = True
-            return self.buf.getvalue()
 
         data = self.buf.getvalue()
         ret, rest = data[:size], data[size:]
-        self.buf.truncate(0)
+        self.buf = six.BytesIO()
         self.buf.write(rest)
         return ret
 
 class Body(object):
     def __init__(self, reader):
         self.reader = reader
-        self.buf = StringIO()
-    
+        self.buf = six.BytesIO()
+
     def __iter__(self):
         return self
-    
-    def next(self):
+
+    def __next__(self):
         ret = self.readline()
         if not ret:
             raise StopIteration()
         return ret
+    next = __next__
 
     def getsize(self, size):
         if size is None:
-            return sys.maxint
-        elif not isinstance(size, (int, long)):
+            return six.MAXSIZE
+        elif not isinstance(size, six.integer_types):
             raise TypeError("size must be an integral type")
         elif size < 0:
-            return sys.maxint
+            return six.MAXSIZE
         return size
-    
+
     def read(self, size=None):
         size = self.getsize(size)
         if size == 0:
-            return ""
+            return b""
 
         if size < self.buf.tell():
             data = self.buf.getvalue()
             ret, rest = data[:size], data[size:]
-            self.buf.truncate(0)
+            self.buf = six.BytesIO()
             self.buf.write(rest)
             return ret
 
@@ -212,53 +215,40 @@ class Body(object):
 
         data = self.buf.getvalue()
         ret, rest = data[:size], data[size:]
-        self.buf.truncate(0)
+        self.buf = six.BytesIO()
         self.buf.write(rest)
         return ret
-    
+
     def readline(self, size=None):
         size = self.getsize(size)
         if size == 0:
-            return ""
-       
-        idx = self.buf.getvalue().find("\n")
-        while idx < 0:
-            data = self.reader.read(1024)
-            if not len(data):
-                break
-            self.buf.write(data)
-            idx = self.buf.getvalue().find("\n")
-            if size < self.buf.tell():
-                break
-        
-        # If we didn't find it, and we got here, we've
-        # exceeded size or run out of data.
-        if idx < 0:
-            rlen = min(size, self.buf.tell())
-        else:
-            rlen = idx + 1
+            return b""
+
+        line = self.buf.getvalue()
+        self.buf = six.BytesIO()
+        if len(line) < size:
+            line += self.reader.read(size - len(line))
+        extra_buf_data = line[size:]
+        line = line[:size]
+
+        idx = line.find(b"\n")
+        if idx >= 0:
+            ret = line[:idx+1]
+            self.buf.write(line[idx+1:])
+            self.buf.write(extra_buf_data)
+            return ret
+        self.buf.write(extra_buf_data)
+        return line
 
-            # If rlen is beyond our size threshold, trim back
-            if rlen > size:
-                rlen = size
-        
-        data = self.buf.getvalue()
-        ret, rest = data[:rlen], data[rlen:]
-        
-        self.buf.truncate(0)
-        self.buf.write(rest)
-        return ret
-    
     def readlines(self, size=None):
         ret = []
         data = self.read()
         while len(data):
-            pos = data.find("\n")
+            pos = data.find(b"\n")
             if pos < 0:
                 ret.append(data)
-                data = ""
+                data = b""
             else:
                 line, data = data[:pos+1], data[pos+1:]
                 ret.append(line)
         return ret
-
